// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef example_diameter_launcher_Launcher_hpp
#define example_diameter_launcher_Launcher_hpp

// Standard
#include <fstream>
#include <string>
#include <map>

// Project
#include <anna/core/core.hpp>
#include <anna/comm/comm.hpp>
#include <anna/time/Date.hpp>
#include <anna/diameter/codec/Message.hpp>

// Process
#include <MyCommunicator.hpp>
#include <MyCounterRecorder.hpp>
#include <anna/testing/TestManager.hpp>


namespace anna {
  namespace timex {
    class Engine;
  }
  namespace diameter {
    namespace codec {
      class Engine;
    }
    namespace comm {
      class OriginHost;
    }
  }
}

class TestManager;
class MyDiameterEntity;
class MyLocalServer;
class MyDiameterEngine;

// OriginHost resources
typedef std::map<std::string, anna::diameter::comm::OriginHost*> origin_hosts_t;
typedef std::map<std::string, anna::diameter::comm::OriginHost*>::const_iterator origin_hosts_it;
typedef std::map<std::string, anna::diameter::comm::OriginHost*>::iterator origin_hosts_nc_it;


class Launcher : public anna::comm::Application {

  // Start time:
  anna::time::Date a_start_time;
  std::string a_initialWorkingDirectory;

  // Core engines:
  MyCommunicator *a_communicator;
  anna::timex::Engine* a_timeEngine;
  MyCounterRecorder *a_counterRecorder;
  anna::Millisecond a_admlMinResolution;
  MyCounterRecorderClock *a_counterRecorderClock;

  // Nodes deployment:
  anna::diameter::comm::OriginHost *a_workingNode;
  anna::diameter::comm::OriginHost *a_operatedHost; // auxiliary for eventOperation

  // comm resources:
  anna::comm::ServerSocket* a_httpServerSocket; // HTTP

  std::string getSignalUSR2InputFile() const throw();
  std::string getSignalUSR2OutputFile() const throw();

  void servicesFromXML(const anna::xml::Node* servicesNode, bool eventOperation) throw(anna::RuntimeException);
  anna::Millisecond checkTimeMeasure(const std::string &parameter, const std::string &value) throw(anna::RuntimeException);
  void initialize() throw(anna::RuntimeException); // HTTP
  void run() throw(anna::RuntimeException);



public:
  Launcher();
  //~Launcher(); TODO

  void loadServices(const std::string & xmlPathFile, bool eventOperation = false) throw(anna::RuntimeException);
  void startServices() throw(anna::RuntimeException);

  bool setWorkingNode(const std::string &name) throw();
  anna::diameter::comm::OriginHost *getOriginHost(const std::string &name) const throw(anna::RuntimeException);
  anna::diameter::comm::OriginHost *getOriginHost(const anna::diameter::codec::Message &message) const throw(anna::RuntimeException);
  bool uniqueOriginHost() const throw();

  // Operated host for communication resources smart assignment ////////////////////////////////////////////////////////////
  void updateOperatedOriginHostWithMessage(const anna::diameter::codec::Message &message) throw(anna::RuntimeException);
  anna::diameter::comm::OriginHost *getWorkingNode() const throw(anna::RuntimeException);
  anna::diameter::comm::OriginHost *getOperatedHost() const throw(anna::RuntimeException);
  MyDiameterEntity *getOperatedEntity() const throw(anna::RuntimeException);
  MyLocalServer *getOperatedServer() const throw(anna::RuntimeException);
  MyDiameterEngine *getOperatedEngine() const throw(anna::RuntimeException);
  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  MyCommunicator *getCommunicator() throw() { return a_communicator; }
  void eventOperation(const std::string &, std::string &) throw(anna::RuntimeException);
  void forceCountersRecord() throw(anna::RuntimeException) { if (a_counterRecorderClock) a_counterRecorderClock->tick(); }
  void logStatisticsSamples(const std::string &conceptsList) throw();


  anna::xml::Node* asXML(anna::xml::Node* parent) const throw();
  void resetStatistics() throw();
  void resetCounters() throw();
  void signalUSR2() throw(anna::RuntimeException);
  std::string help() const throw();
  anna::xml::Node* oamAsXML(anna::xml::Node* parent) const throw();
  anna::xml::Node* statsAsXML(anna::xml::Node* parent) const throw();

  // helpers
  bool getDataBlockFromHexFile(const std::string &pathfile, anna::DataBlock &db) const throw(anna::RuntimeException);
  bool getContentFromFile(const std::string &pathfile, std::string &content) const throw(anna::RuntimeException);

  friend class TestManager;
};

#endif
