// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <functional>

#include <anna/core/core.hpp>
#include <anna/xml/xml.hpp>

#include <anna/app/Application.hpp>

class Test : public app::Application {
public:
   Test ();
   
private:
   void run () throw (anna::RuntimeException);      
};

using namespace std;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Test test;

   xml::functions::initialize ();
   
   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug);
      Logger::initialize ("XML", new TraceWriter ("file.trace", 2048000));

      test.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

Test::Test () : 
   app::Application ("testfunctions", "Checking xml binary", "1.0") 
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("xml", CommandLine::Argument::Mandatory, "XML document");
}

void Test::run () 
   throw (RuntimeException)
{
   CommandLine& commandLine (CommandLine::instantiate ());

   xml::DocumentFile xmlDoc;

   xmlDoc.initialize (commandLine.getValue ("xml"));

   xml::Compressor compressor;
   const DataBlock& dataBlock = compressor.apply (xmlDoc);

   Logger::debug (anna::functions::asString (dataBlock, 24), ANNA_FILE_LOCATION);
   cout << "Compression: " << anna::functions::asString (dataBlock, 24) << endl;
   cout << dataBlock.getSize () << " bytes" << endl << endl;

   xml::Decompressor decompressor;
   const xml::Document& xmlDoc2 = decompressor.apply (dataBlock);
   cout << "DeCompression: " << anna::functions::asString (xmlDoc2.getContent (), 24) << endl;

}

