// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_Application_hpp
#define anna_comm_Application_hpp

#include <vector>

#include <anna/app/Application.hpp>

#include <anna/comm/functions.hpp>

namespace anna {

namespace xml {
class Node;
}

namespace comm {

class Communicator;
class TransportFactory;

/**
   Clase que modela el cargador de configuracin de gestor de comunicaciones.

   Esta clase realiza dos operaciones primordiales para el comunicador:
   \li Establece las direcciones en las que el comunicador de nuestra aplicacion atiende peticiones
   de los procesos clientes. Este paso slo sera necesario si nuestro proceso acta como servidor
   de algn otro proceso.
   \li Establece las direcciones de los servidores de nuestra aplicacion. Este paso slo sera necesario
   si nuestra aplicacion acta como cliente de algn otro servidor.

   Hay que destacar que nuestra aplicacion puede actuar simultaneamente como cliente y/o servidor por
   lo que puede haber ocasiones en que sea requieran las dos operaciones.

   \see Communicator::setup
*/
class Application : public app::Application {
public:
  /**
     Devuelve la instancia de la factoria de protocolos a usar por defecto en esta
     aplicacion.
     \return La instancia de la factoria de protocolos a usar por defecto en esta
     aplicacion.
  */
  virtual TransportFactory& getDefaultTransportFactory() throw();

protected:
  /**
     Constructor.

     @param shortName Nombre lgico de esta instancia. Ver Runnable.
     @param title titulo de la aplicacion que aparecera al arrancar.
     @param version Version de este programa. Aconsejamos el forma X.YRZZ. Donde X es la
     version principal, Y la version secundaria y ZZ es el nmero de entrega realizada.
     \param date Fecha de realizacion del componente. Normalmente sera el contenido de la macro __DATE__.
     \param time Hora de realizacion del componente. Normalmente sera el contenido de la macro __TIME__.
  */
  Application(const char *shortName, const char *title, const char *version, const char* date = NULL, const char* time = NULL);

  /**
     Metodo manejador que podemos re-escribir para tratar la recepcion de la senhal SIGTERM.
  */
  virtual void signalTerminate() throw(RuntimeException);

private:
  Application(const Application&);
  bool supportCommunication() const throw() { return true; }

  friend class Communicator;
  friend anna::comm::Application& comm::functions::getApp() throw(RuntimeException);
};

}
}

#endif
