// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_xml_Namespace_hpp
#define anna_xml_Namespace_hpp

#include <string>

#include <anna/core/Allocator.hpp>

namespace anna {

namespace xml {

class Node;

/**
   Clase que modela los Namespaces que puede contener un documento XML.

   Un namespace XML estar definido por un nombre y por una URI de la que obtener las definiciones.

   La lista de namespaces asociados a un documento analizado estar asociado al nodo 'root' del
   documento, aunque el resto de los nodos podr hacer referencia al mismo.
*/
class Namespace {
public:
  /**
   * Devuelve el nombre de este namespace.
   * \return  el nombre de este namespace.
   */
  const std::string& getName() const throw() { return a_name; }

  /**
   * Devuelve la referencia asociada a este namespace.
   * \return la referencia asociada a este namespace.
   */
  const std::string& getReference() const throw() { return a_reference; }

  /**
     Devuelve una cadena con toda la informacin relevante de esta instancia.
     \return Una cadena con toda la informacin relevante de esta instancia.
  */
  std::string asString() const throw();

private:
  /**
     Constructor.
  */
  Namespace() {;}

  std::string a_name;
  std::string a_reference;

  void setName(const char* name) throw() { a_name = name; }
  void setName(const std::string& name) throw() { a_name = name; }
  void setReference(const char* reference) throw() { a_reference = reference; }

  friend class Allocator<Namespace>;
  friend class Node;
};

}
}

#endif
