// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <netdb.h>

#include <anna/core/tracing/Logger.hpp>

#include <anna/comm/Network.hpp>

#include <anna/comm/Device.hpp>
#include <anna/comm/Host.hpp>
#include <anna/comm/Server.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

using namespace std;
using namespace anna;

comm::Device* comm::Network::find(const in_addr_t& address)
throw() {
  Device* result = NULL;

  if(a_cacheDevice != NULL && *a_cacheDevice == address)
    return a_cacheDevice;

  for(device_iterator ii = device_begin(), maxii = device_end(); ii != maxii; ii ++) {
    result = device(ii);

    if(*result == address)
      return a_cacheDevice = result;
  }

  a_devices.push_back(result = new Device(address));
  return a_cacheDevice = result;
}

comm::Host* comm::Network::find_host(const char* name)
throw() {
  Host* result = NULL;

  if(a_cacheHost != NULL && anna_strcmp(a_cacheHost->getName().c_str(), name) == 0)
    return a_cacheHost;

  for(host_iterator ii = host_begin(), maxii = host_end(); ii != maxii; ii ++) {
    result = host(ii);

    if(anna_strcmp(result->getName().c_str(), name) == 0)
      return a_cacheHost = result;
  }

  a_hosts.push_back(result = new Host(name));
  return a_cacheHost = result;
}

comm::Host* comm::Network::resolve(const char* hostname)
throw(RuntimeException) {
  comm::Host* result = find_host(hostname);
  struct hostent *host;

  if((host = gethostbyname(hostname)) == NULL) {
    string msg("comm::Network::resolve | Host: ");
    msg += hostname;
    msg += " | ";
    msg += hstrerror(h_errno);
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(host->h_addrtype != AF_INET) {
    string msg("comm::Network::resolve | Host: ");
    msg += hostname;
    msg += " | Address type unsupported";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  for(int i = 0; host->h_addr_list[i] != NULL; i ++) {
    const in_addr_t& address = *reinterpret_cast <in_addr_t*>(host->h_addr_list[i]);
    LOGDEBUG(
      string msg("comm::Network::resolve | Host: ");
      msg += hostname;
      msg += " | IP: ";
      msg += inet_ntoa(*reinterpret_cast <in_addr*>(host->h_addr_list[i]));
      Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    result->assign(find(address));
  }

  return result;
}


comm::Server* comm::Network::createServer(const char* ip, const int remotePort, const bool autoRecovery, comm::TransportFactory* transportFactory, const Port::_v mode, const DoConnect::_v doConnect)
throw(RuntimeException) {
  comm::Server* result(NULL);
  comm::Host* host = find_host(ip);
  Guard guard(host, "comm::Host from comm::Network::createServer");

  if(mode == Port::Unique) {
    /**
     * Si ya existe una conexin previa con esa (ip, port) la devuelve
     */
    if((result = host->find_server(remotePort)) != NULL)
      return result;
  }

  comm::Device* device = find(comm::Device::asAddress(ip));
  host->assign(device);
  string serverName = ip;
  serverName += functions::asText(":", remotePort);
  return host->createServer(serverName, remotePort, autoRecovery, transportFactory, false /* ignore incoming messages */, (doConnect == DoConnect::Yes));
}

comm::Server* comm::Network::createServer(const char* ip, const int remotePort, const bool autoRecovery, comm::ReceiverFactory& rrff, comm::TransportFactory* transportFactory, const Port::_v mode, const DoConnect::_v doConnect)
throw(RuntimeException) {
  Server* result = createServer(ip, remotePort, autoRecovery, transportFactory, mode, doConnect);
  result->setReceiverFactory(rrff);
  return result;
}

//comm::Server* comm::Network::findServer (const char* ip, const int remotePort)
//   throw (RuntimeException)
//{
//   comm::Host* _host (NULL);
//
//   for (host_iterator ii = host_begin (), maxii = host_end (); ii != maxii; ii ++) {
//      if (host (ii)->getName () == ip) {
//         _host = host (ii);
//         break;
//      }
//   }
//
//   return (_host == NULL) ? NULL: _host->find_server (remotePort);
//}


comm::Server* comm::Network::resolveServer(const char* hostname, const int remotePort, const bool autoRecovery, comm::TransportFactory* transportFactory, const Port::_v mode, const DoConnect::_v doConnect)
throw(RuntimeException) {
  comm::Server* result(NULL);
  comm::Host* host = resolve(hostname);
  Guard guard(host, "comm::Host from comm::Network::resolveServer");

  if(mode == Port::Unique) {
    /**
     * Si ya existe una conexin previa con esa (ip, port) la devuelve
     */
    if((result = host->find_server(remotePort)) != NULL)
      return result;
  }

  string serverName = hostname;
  serverName += functions::asText(":", remotePort);
  return host->createServer(serverName, remotePort, autoRecovery, transportFactory, false /* ignore incoming messages */, (doConnect == DoConnect::Yes));
}

comm::Server* comm::Network::resolveServer(const char* hostname, const int remotePort, const bool autoRecovery, comm::ReceiverFactory& rrff, comm::TransportFactory* transportFactory, const Port::_v mode, const DoConnect::_v doConnect)
throw(RuntimeException) {
  Server* result = resolveServer(hostname, remotePort, autoRecovery, transportFactory, mode, doConnect);
  result->setReceiverFactory(rrff);
  return result;
}

comm::INetAddress comm::Network::getINetAddress(const char* ip, const int port)
throw(RuntimeException) {
  const Device* device = find(Device::asAddress(ip));
  return INetAddress(device, port);
}

comm::INetAddress comm::Network::getINetAddress(const std::string& ip, const int port)
throw(RuntimeException) {
  const Device* device = find(Device::asAddress(ip));
  return INetAddress(device, port);
}

xml::Node* comm::Network::asXML(xml::Node* parent) const
throw() {
  xml::Node* result = parent->createChild("comm.Network");
  xml::Node* node = result->createChild("comm.Devices");

  for(const_device_iterator ii = device_begin(), maxii = device_end(); ii != maxii; ii ++)
    device(ii)->asXML(node);

  node = result->createChild("comm.Hosts");

  for(const_host_iterator ii = host_begin(), maxii = host_end(); ii != maxii; ii ++)
    host(ii)->asXML(node);

  return result;
}

