// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/comm/DatagramSocket.hpp>
#include <anna/comm/Communicator.hpp>
#include <anna/comm/Device.hpp>
#include <anna/comm/Transport.hpp>
#include <anna/comm/Receiver.hpp>

#include <anna/comm/handler/DatagramSocket.hpp>

using namespace std;
using namespace anna;

void comm::handler::DatagramSocket::initialize()
throw(RuntimeException) {
  if(a_datagramSocket == NULL) {
    string msg(asString());
    msg += " | comm::DatagramSocket was not established";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(a_datagramSocket->isConnected() == false)
    a_datagramSocket->connect();

  setfd(a_datagramSocket->getfd());
}

comm::ClientSocket* comm::handler::DatagramSocket::getClientSocket()
throw() {
  return a_datagramSocket;
}

void comm::handler::DatagramSocket::apply()
throw(RuntimeException) {
  Guard guard(a_datagramSocket, "anna::comm::DatagramSocket from anna::comm::handler::DatagramSocket::apply");
  comm::DatagramSocket::Notify::_v notify = a_datagramSocket->receive();

  if(notify == comm::DatagramSocket::Notify::Corrupt) {
    a_datagramSocket->forgot();
    return;
  }

  if(notify != comm::DatagramSocket::Notify::ReceiveData)
    return;

  const Message* message;
  const DataBlock* dataBlock;
  Transport* transport = a_datagramSocket->getTransport();
  Receiver* receiver = a_datagramSocket->getReceiver();

  while((dataBlock = a_datagramSocket->fetch()) != NULL && canContinue()) {
    try {
      try {
        message = transport->decode(*dataBlock);
      } catch(RuntimeException& ex) {
        a_datagramSocket->forgot();
        ex.trace();
        message = NULL;
        break;
      }

      if(receiver == NULL)
        a_communicator->eventReceiveMessage(*a_datagramSocket, *message);
      else
        receiver->apply(*a_datagramSocket, *message);
    } catch(RuntimeException& ex) {
      ex.trace();
    }
  }
}

/*
 * Se invoca desde Communicator::detach.
 *
 * [Tx] -> Communicator
 */
void comm::handler::DatagramSocket::finalize()
throw() {
  if(a_datagramSocket == NULL)
    return;

  a_datagramSocket->close();
  a_datagramSocket = NULL;
}

string comm::handler::DatagramSocket::asString() const
throw() {
  string result("comm::handler::DatagramSocket { ");
  result += comm::Handler::asString();
  result += " | ";
  result += functions::asString(a_datagramSocket);
  return result += " }";
}

xml::Node* comm::handler::DatagramSocket::asXML(xml::Node* parent) const
throw() {
  xml::Node* result = parent->createChild("comm.handler.DatagramSocket");
  comm::Handler::asAttribute(result);

  if(a_datagramSocket)
    a_datagramSocket->asXML(result);

  return result;
}


