// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <time.h>
#include <sys/time.h>
#include <stdlib.h>

#include <anna/core/functions.hpp>
#include <anna/core/util/Millisecond.hpp>
#include <anna/core/util/Second.hpp>
#include <anna/core/util/Microsecond.hpp>

using namespace std;
using namespace anna;

#define implement_operator(op) \
   bool Millisecond::operator op (const Second& other) const \
      throw ()\
   {\
      return a_value op (other.a_value * (type_t)1000);\
   }\
   bool Millisecond::operator op (const Microsecond& other) const\
      throw ()\
   {\
      return a_value op (other.a_value / 1000);\
   }

Millisecond::Millisecond(const Second& other) : a_value(other.a_value) { a_value *= (type_t)1000;}

Millisecond::Millisecond(const Microsecond& other) : a_value(other.a_value / 1000) {;}

Millisecond& Millisecond::operator= (const Second & other)
throw() {
  a_value = other.a_value;
  a_value *= (type_t)1000;
  return *this;
}

Millisecond& Millisecond::operator= (const Microsecond & other)
throw() {
  a_value = (other.a_value / 1000);
  return *this;
}

implement_operator( ==)
implement_operator( !=)
implement_operator( >)
implement_operator( <)

//static
Millisecond Millisecond::getTime()
throw() {
  struct timeval tv;
  gettimeofday(&tv, NULL);
  Millisecond result(Second(tv.tv_sec));
  Millisecond aux(Microsecond(tv.tv_usec));
  result.a_value += aux.a_value;
  return result;
}

timeval* Millisecond::getTimeVal(timeval& tv) const
throw() {
  if(a_value < 0)
    return NULL;

  tv.tv_sec = a_value / 1000;
  tv.tv_usec = (a_value % 1000) * (type_t)1000;
  return &tv;
}

string Millisecond::asString() const
throw() {
  string result(functions::asString(a_value));
  return result += " ms";
}

//static
Millisecond Millisecond::fromString(const std::string& value)
throw(RuntimeException) {
  if(value.find(" ms") == string::npos) {
    string msg("String: ");
    msg += value;
    msg += " | Invalid expression for Millisecond";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return Millisecond(atoll(value.c_str()));
}

