// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>

#include <anna/core/functions.hpp>
#include <anna/core/util/Second.hpp>
#include <anna/core/util/Millisecond.hpp>
#include <anna/core/util/Microsecond.hpp>

using namespace std;
using namespace anna;

#define implement_operator(op) \
   bool Second::operator op (const Millisecond& other) const \
      throw ()\
   {\
      return a_value op (other.a_value / 1000);\
   }\
   bool Second::operator op (const Microsecond& other) const\
      throw ()\
   {\
      return a_value op (other.a_value / 1000000);\
   }

Second::Second(const Millisecond& other) : a_value(other.a_value / 1000) {;}

Second::Second(const Microsecond& other) : a_value(other.a_value / 1000000) {;}

Second& Second::operator= (const Millisecond & other)
throw() {
  a_value = (other.a_value / 1000);
  return *this;
}

Second& Second::operator= (const Microsecond & other)
throw() {
  a_value = (other.a_value / 1000000);
  return *this;
}

implement_operator( ==)
implement_operator( !=)
implement_operator( >)
implement_operator( <)

string Second::asDateTime(const char* format) const
throw() {
  char aux [DateTimeSizeString];
  return string(asDateTime(aux, format));
}

const char* Second::asDateTime(char* result, const char* format) const
throw() {
  struct tm* tt = localtime((time_t*) & a_value);
  char aux [256];

  if(strftime(aux, sizeof(aux), format, tt) == 0)
    anna_strcpy(aux, "Bad date");

  return anna_strcpy(result, aux);
}

//static
Second Second::getTime()
throw() {
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return Second(tv.tv_sec);
}

//static
Second Second::getLocalTime()
throw() {
  return Second(time(NULL));
}

string Second::asString() const
throw() {
  string result(functions::asString(a_value));
  return result += " sec";
}

//static
Second Second::fromString(const std::string& value)
throw(RuntimeException) {
  if(value.find(" sec") == string::npos) {
    string msg("String: ");
    msg += value;
    msg += " | Invalid expression for Second";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return Second(atoi(value.c_str()));
}


