// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter.comm/Response.hpp>
#include <anna/diameter.comm/Message.hpp>
#include <anna/diameter.comm/TimerManager.hpp>
#include <anna/diameter/defines.hpp>
#include <anna/diameter/helpers/helpers.hpp>

#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>



using namespace std;
using namespace anna;

diameter::comm::Response::response_pool diameter::comm::Response::st_responses;

//----------------------------------------------------------------------------------------
// Se invocan desde diameter::comm::Session
//----------------------------------------------------------------------------------------
diameter::comm::Response* diameter::comm::Response::instance(const ClassCode::_v & classCode, const HopByHop hopByHop)
throw(anna::RuntimeException) {
  diameter::comm::Response* result = st_responses.create();
  result->a_classCode = classCode;
  result->a_hopByHop = hopByHop;
  result->clear();
  LOGDEBUG(
    string msg("anna::diameter::comm::Response::instance | ");
    msg += result->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return result;
}

void diameter::comm::Response::release(diameter::comm::Response* response)
throw() {
  try {
    st_responses.release(response);
  } catch(anna::RuntimeException& ex) {
    ex.trace();
  }
}

diameter::comm::Response::Response() :
  a_classCode(ClassCode::Undefined),
  a_hopByHop(0),
  a_session(NULL),
  a_timer(NULL),
  a_request(NULL) {
}

void diameter::comm::Response::clear()
throw() {
  a_resultCode = ResultCode::Undefined;
  a_session = NULL;
  a_timer = NULL;
  a_request = NULL;
}

void diameter::comm::Response::activateTimer()
throw(anna::RuntimeException) {
  a_timer = TimerManager::instantiate().createTimer(this);
}

void diameter::comm::Response::cancelTimer()
throw() {
  if(a_timer != NULL) {
    try {
      TimerManager::instantiate().cancelTimer(a_timer);
    } catch(anna::RuntimeException& ex) {
      ex.trace();
    }

    a_timer = NULL;
  }
}

const char* diameter::comm::Response::asText(const ResultCode::_v rc)
throw() {
  static const char* text [] = { "Undefined", "Success", "Timeout", "DiameterUnavailable" };
  return text [rc];
}

//bool diameter::comm::Response::isKeepAlive() const
//throw() {
//   if (a_request) {
//      if (a_request->getCommandId() == diameter::helpers::base::COMMANDID__Device_Watchdog_Request)
//         return true;
//   }
//
//   return false;
//}

string diameter::comm::Response::asString() const
throw() {
  string result("diameter::comm::Response { ClassCode: ");
  result += ClassCode::asText(a_classCode);
  result += anna::functions::asString(" | HopByHop: %u", a_hopByHop);
  result += " | ResultCode: ";
  result += asText(a_resultCode);
  return result += " }";
}

