// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter.comm/TimerManager.hpp>
#include <anna/diameter.comm/Response.hpp>
#include <anna/diameter.comm/Session.hpp>

#include <anna/core/tracing/Logger.hpp>
#include <anna/app/functions.hpp>
#include <anna/timex/Engine.hpp>


using namespace std;
using namespace anna::diameter::comm;


TimerManager::TimerManager() :
  anna::timex::TimeEventObserver("anna::diameter::comm::TimerManager"),
  a_timeController(NULL) {
}

//-------------------------------------------------------------------------------------------------------
// (1) Bloquea el TimerManager el primero para mantener siempre el mismo orden de acceso a la
// seccion critica, lo que evita interbloqueos.
//-------------------------------------------------------------------------------------------------------
Timer* TimerManager::createTimer(Session* session, const anna::diameter::comm::Timer::Type::_v type)
throw(anna::RuntimeException) {
  Timer* result(NULL);

  if(a_timeController == NULL)
    a_timeController = anna::app::functions::component <anna::timex::Engine> (ANNA_FILE_LOCATION);

  anna::Guard guard(a_timeController, "anna::diameter::comm::TimerManager::createTimer");              // (1)
  result = a_timers.create();
  result->setType(type);
  result->setId((anna::timex::TimeEvent::Id) session);
  result->setObserver(this);
  result->setContext(session);
  //Timeout depends on type:
  // - SessionUnbind: temporizador de cierre local (2*Tx) como proteccion
  // - SessionRecover: de momento no lo estamos usando, quiza por ello, el activateActionTimer debiera llamarse activateUnbindTimer...
  result->setTimeout((anna::Millisecond)(2 * session->getClassCodeTimeout(ClassCode::ApplicationMessage).getValue()));
  LOGDEBUG(
    string msg("anna::diameter::comm::TimerManager::createTimer (actionTimer) | ");
    msg += result->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_timeController->activate(result);
  return result;
}


Timer* TimerManager::createTimer(Response* response)
throw(anna::RuntimeException) {
  Timer* result(NULL);

  if(a_timeController == NULL)
    a_timeController = anna::app::functions::component <anna::timex::Engine> (ANNA_FILE_LOCATION);

  anna::Guard guard(a_timeController, "anna::diameter::comm::TimerManager::createTimer");              // (1)
  result = a_timers.create();
  const ClassCode::_v v = response->getClassCode();
  result->setType(Timer::Type::ResponseExpiration);
  result->setId((anna::timex::TimeEvent::Id) response);
  result->setObserver(this);
  result->setContext(response);
  result->setTimeout(response->getSession()->getClassCodeTimeout(v));
  // DWR doesn't arrive here: no context manage for this message
  //if (response->isKeepAlive()) result->setTimeout(response->getSession()->getTimeout());
  LOGDEBUG(
    string msg("anna::diameter::comm::TimerManager::createTimer (response) | ");
    msg += result->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_timeController->activate(result);
  return result;
}


Timer* TimerManager::createTimer(LocalServer* localServer)
throw(anna::RuntimeException) {
  Timer* result(NULL);

  if(a_timeController == NULL)
    a_timeController = anna::app::functions::component <anna::timex::Engine> (ANNA_FILE_LOCATION);

  anna::Guard guard(a_timeController, "anna::diameter::comm::TimerManager::createTimer");              // (1)
  result = a_timers.create();
  result->setType(Timer::Type::LocalServerAttach);
  result->setId((anna::timex::TimeEvent::Id) localServer);
  result->setObserver(this);
  result->setContext(localServer);
  result->setTimeout((anna::Millisecond)5000); // inactivityTime / 2 perhaps ??
  LOGDEBUG(
    string msg("anna::diameter::comm::TimerManager::createTimer (attachPlanning) | ");
    msg += result->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_timeController->activate(result);
  return result;
}


void TimerManager::cancelTimer(Timer* timer)
throw() {
  if(timer == NULL)
    return;

  LOGDEBUG(
    string msg("anna::diameter::comm::TimerManager::cancel | ");
    msg += timer->asString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );

  try {
    if(a_timeController == NULL)
      a_timeController = anna::app::functions::component <anna::timex::Engine> (ANNA_FILE_LOCATION);

    a_timeController->cancel(timer);
  } catch(anna::RuntimeException& ex) {
    ex.trace();
  }
}

//------------------------------------------------------------------------------------------
// Se invoca automaticamente desde anna::timex::Engine
//------------------------------------------------------------------------------------------
void TimerManager::release(anna::timex::TimeEvent* timeEvent)
throw() {
  Timer* timer = static_cast <Timer*>(timeEvent);
  timer->setContext(NULL);
  a_timers.release(timer);
}

