// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <ctype.h>

#include <anna/config/defines.hpp>
#include <anna/core/DataBlock.hpp>

#include <anna/http/internal/Tokenizer.hpp>
#include <anna/http/internal/defines.hpp>
#include <anna/http/functions.hpp>

using namespace anna;

//--------------------------------------------------------------------------------------------
// Extrae los datos del bloque de memoria separando por espacios en blancos, tabs, etc, etc
//--------------------------------------------------------------------------------------------
void http::Tokenizer::apply(const DataBlock& data)
throw(RuntimeException) {
  const char* p = data.getData();
  const char* maxp = p + data.getSize();
  bool searchingInit = true;
  const char* init = NULL;
  int len = 0;
  clear();

  while(p < maxp) {
    if(searchingInit == true) {
      if(isSpace(*p) == false) {
        init = p;
        len = 1;
        searchingInit = false;
      }
    } else {
      if(isSpace(*p) == true) {
        createToken(init, len);
        searchingInit = true;
      } else
        len ++;
    }

    p ++;
  }

  if(searchingInit == false)
    createToken(init, len);
}

void http::Tokenizer::apply(const DataBlock& data, const char* separator)
throw(RuntimeException) {
  const char* p = data.getData();
  int size = data.getSize();
  const char* maxp = p + size;
  const int lenSeparator = anna_strlen(separator);
  int pos;
  clear();

  while(p < maxp && size > 0) {
    if((pos = find(p, size, separator)) != -1) {
      createToken(p, pos);
      p += (pos + lenSeparator);
      size -= (pos + lenSeparator);
    } else {
      createToken(p, size);
      p += size;
      size = 0;
    }
  }
}

void http::Tokenizer::apply(const DataBlock& data, const char separator)
throw(RuntimeException) {
  const char* p = data.getData();
  int size = data.getSize();
  const char* maxp = p + size;
  int pos;
  clear();

  while(p < maxp && size > 0) {
    if((pos = find(p, size, separator)) != -1) {
      createToken(p, pos);
      p += (pos + 1);
      size -= (pos + 1);
    } else {
      createToken(p, size);
      p += size;
      size = 0;
    }
  }
}

const http::Token* http::Tokenizer::operator [](int index) const
throw() {
  const_iterator ii = begin();
  const_iterator maxii = end();

  while(index && ii != maxii) {
    ii ++;
    index --;
  }

  return operator[](index);
}

int http::Tokenizer::find(const char* data, const int size, const char searched)
throw() {
  for(int i = 0; i < size; i ++, data ++) {
    if(*data == searched)
      return i;
  }

  return -1;
}

int http::Tokenizer::find(const char* data, const int size, const char* searched)
throw() {
  static const int EndOfLineLen = 2;
  int result(-1);
  const char* w(data);
  int s(size);
  int pos;
  const int slen = anna_strlen(searched);

  if(slen == EndOfLineLen) {
    while((pos = find(w, s, *searched)) != -1) {
      if((s - pos) < EndOfLineLen)
        break;

      w += pos;

      if(*w == *searched && *(w + 1) == *(searched + 1)) {
        result = (w - data);
        break;
      }

      w ++;
      s -= pos;
    }
  } else {
    while((pos = find(w, s, *searched)) != -1) {
      if((s - pos) < slen)
        break;

      w += pos;

      if(anna_strncmp(w, searched, slen) == 0) {
        result = (w - data);
        break;
      }

      w ++;
      s -= pos;
    }
  }

  return result;
}

