// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>

#include <anna/http/wims20/ClientSide.hpp>

#include <anna/http/Request.hpp>

using namespace std;
using namespace anna;

/*
 * Los niveles parece que sern bastante estables, as que nos ahorramos calcular toda su ruta
 * cada vez que tengamos que calcular la URI.
 */
void http::wims20::ClientSide::addOtherLevel(const std::string& otherLevel)
throw(RuntimeException) {
  if(a_strOtherLevels == NULL)
    a_strOtherLevels = Abstract::createString(otherLevel);
  else {
    *a_strOtherLevels += '/';
    *a_strOtherLevels += otherLevel;
  }

  Abstract::other_level_add(otherLevel);
}

/*
   http://domain-openapis/path-openapis/serviceID/guid/other_possible_levels?query_parameters

   Dnde los campos tienen siguen la siguiente especificacin:
   \li http://domain-openapis: Identifica el recurso del Open API.
   \li path-openapis: Recurso opcional que ajusta la ruta hacia los recursos de ste API.
   \li serviceID: Identificador de recurso.
   \li guid: Identificador del usuario que solicita la peticin.
   \li other_possible_level: Opcionalmente se pueden indicar tantos niveles jerrquicos como fuera
   necesario para el servicio.
   \li query_parameters: Lista de parmetros. Si hay ms de un parmetro se separar con '&'.
 */
void http::wims20::ClientSide::codeOn(http::Request& request)
throw(RuntimeException) {
  a_uri = Abstract::calculeFixedPart();

  if(Abstract::hasOtherLevels())
    Abstract::appendWithSlash(a_uri, *a_strOtherLevels);

  if(Abstract::hasParameters()) {
    a_uri += '?';
    bool first = true;

    for(parameter_iterator ii = parameter_begin(), maxii = parameter_end(); ii != maxii; ii ++) {
      if(first == false)
        a_uri += '&';

      a_uri += *Abstract::parameter_name(ii);
      a_uri += '=';
      a_uri += *Abstract::parameter_value(ii);
      first = false;
    }
  }

  LOGDEBUG(
    string msg("http::wims20::ServerSide::codeOn | ");
    msg += asString();
    msg += " | Result (URI): ";
    msg += a_uri;
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  request.setURI(a_uri);
}
